#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include "ports.h"
#include "eiuhr.h"
#include "countdown.h"

static inline void beeper (uint8_t on);

// Die leeren ISRs dienen nur zum Aufwecken
EMPTY_INTERRUPT (SIG_INTERRUPT0)
EMPTY_INTERRUPT (SIG_INTERRUPT1)

void standby(void)
{
	// Schlafen vorbereiten
    // Keine IRQs
	cli();

    // Alle Ports = IN + PULLUP    
	DDRA = 0; 	PORTA = 0xff;
	DDRB = 0; 	PORTB = 0xff;
	DDRD = 0; 	PORTD = 0xff;

	// Peripherie deaktivieren
	TIMSK = 0;
	DIDR  = (1 << AIN0D) | (1 << AIN1D);
    
    // INT IRQ zum Aufwecken
    GIMSK = (1 << INT0)  | (1 << INT1);
	EIFR  = (1 << INTF0) | (1 << INTF1);
	
    // Schlafen
	sleep_enable();
	set_sleep_mode (SLEEP_MODE_STANDBY);
	
	sei();
	sleep_cpu();
	
    // Aufwachen
	cli();
	sleep_disable();
    
    // Externe IRQs deaktivieren
	GIMSK = 0;
    
    // Peripherie initialisieren
	DIDR  = 0;
    timer_init();
    
	sei();
}

void timer_init(void)
{
	// CTC: Mode #4 for Timer1
	// und full MCU clock
	
	// OutputCompare for desired Timer1 frequency
	OCR1A = (unsigned short) ((unsigned long) F_CPU / IRQS_PER_SECOND-1);
	
	// Timer1 ist counter: Clear Timer on Compare Match (CTC, Mode #4)
	// Timer1 runs @ F_CPU: prescale = 1
	TCCR1B = (1 << WGM12) | (1 << CS10);
    
	TIMSK = 1 << OCIE1A;
}

void beeper (uint8_t on)	
{
	if (on)
	{
		// Beeper an
		MAKE_OUT (PORT_BEEPA);
		MAKE_OUT (PORT_BEEPB);
        // PWM: Mode #3 for Timer0
        // dutyA = dutyB = 50%
        OCR0A = 128;
        OCR0B = 128;
        // Prescale = 1 (Timer0 @ 1MHz) --> PWM @ 3.9kHz
        TCCR0A = (1 << WGM01) | (1 << WGM00) | (1 << COM0A1) | (1 << COM0B1) | (1 << COM0B0);
		TCCR0B = 1 << CS00;
	}
	else
	{
		// Beeper aus
		MAKE_IN (PORT_BEEPA);
		MAKE_IN (PORT_BEEPB);
		SET (PORT_BEEPA);
		SET (PORT_BEEPB);
		TCCR0A = 0;
		TCCR0B = 0;
	}
}

/*
      A1 A2 A3 A4 A5
  K1   1  3  4  5  7
  K2  10 20 40  6  8
  K3   2 30 50  /  9
*/
void eiuhr_set_leds (uint8_t kathode)
{
	static uint8_t skip;
	
	uint8_t  z, zehner = eiuhr.bits_zehner;
	uint16_t e, einer  = eiuhr.bits_einer;

	#define SET_10(b,p) if (z = zehner & (1 << (b/10-1)), z) SET(PORT_ ## p)
	#define SET_1(b,p)  if (e = einer  & (1 << (b-1)), e)    SET(PORT_ ## p)
	
	if (0 == kathode)
		if (++skip == 3)
			skip = 0;

	if (!eiuhr.hell && skip != kathode)
    {
        // Gelbe Stunden-LED etwas heller als die roten LEDs
        if (2 == kathode && 0 == skip)        
        {
			SET_10 (60, A4);
            CLR (PORT_K3);
        }
    
		return;			
    }    
	
	if (0 == kathode)
	{
		SET_1 (1, A1);
		SET_1 (3, A2);
		SET_1 (4, A3);
		SET_1 (5, A4);
		SET_1 (7, A5);
		
		CLR (PORT_K1);
	}
	
	if (1 == kathode)
	{
		SET_10 (10, A1);
		SET_10 (20, A2);
		SET_10 (40, A3);
		SET_1  ( 6, A4);
		SET_1  ( 8, A5);
        
		CLR (PORT_K2);
	}
	
	if (2 == kathode)
	{
		SET_1  ( 2, A1);
		SET_10 (30, A2);
		SET_10 (50, A3);
		SET_10 (60, A4);
		SET_1  ( 9, A5);
		
		CLR (PORT_K3);
	}
}

#include "eiuhr.c"
