#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include "ports.h"
#include "eiuhr.h"
#include "countdown.h"

#define F_SPEAKER 3900

static inline void beeper (uint8_t on);

// Die leere ISR dient nur zum Aufwecken
EMPTY_INTERRUPT (SIG_PIN_CHANGE0)

void standby(void)
{
	// Schlafen vorbereiten
    // Keine IRQs
	cli();

    // Alle Ports = IN + PULLUP    
	DDRA = 0; 	PORTA = 0xff;
	DDRB = 0; 	PORTB = 0xff;
    
    // PinChange IRQ 0+1 zum Aufwecken
	GIMSK = (1 << PCIE0);
	GIFR  = (1 << PCIF0);
    PCMSK0= (1 << PCINT0) | (1 << PCINT1);

	// Peripherie deaktivieren
	TIMSK0 = 0;
    PRR = (1 << PRUSI) | (1 << PRADC) | (1 << PRTIM1) | (1 << PRTIM0);

    // Schlafen
	sleep_enable();
	set_sleep_mode (SLEEP_MODE_PWR_DOWN);
	
    sei();
    sleep_cpu();

    // Aufwachen
    cli();
	sleep_disable();
    
    // Externe IRQs deaktivieren
	GIMSK = 0;

    // Peripherie initialisieren
    PRR = (1 << PRUSI) | (1 << PRADC);
    timer_init();
    MAKE_IN (PORT_BEEPB);
    CLR     (PORT_BEEPB);
	sei();
}

void timer_init(void)
{
	// Mode #2 (CTC) for Timer0
	// Timer0 runs @ F_CPU: prescale = 1
	TCCR0A = (1 << WGM01);
	TCCR0B = (1 << CS00);
	
	// OutputCompare for desired Timer0 frequency
	OCR0A = (unsigned short) ((unsigned long) F_CPU / IRQS_PER_SECOND-1);
	TIMSK0 = 1 << OCIE0A;
}

void beeper (uint8_t on)	
{
	if (on)
	{
		// Beeper an
		MAKE_OUT (PORT_BEEPA);
		MAKE_OUT (PORT_BEEPB);

        // PWM: Mode #8 for Timer1
        // Prescale = 1 --> PWM @ <F_SPEAKER> kHz
        TCCR1A = (1 << COM1B1) | (1 << COM1A1) | (1 << COM1A0);
		TCCR1B = (1 << WGM13) | (1 << CS10);
       
        const uint16_t icr = 
            (unsigned short) ((unsigned long) F_CPU / (2*F_SPEAKER) -1);
            
        ICR1 = icr;
        // dutyA = dutyB = 50%
        OCR1A = icr/2;
        OCR1B = icr/2;
	}
	else
	{
		// Beeper aus
		MAKE_IN (PORT_BEEPA);
		MAKE_IN (PORT_BEEPB);
		SET (PORT_BEEPA);
		CLR (PORT_BEEPB);
        TCCR1A = 0;
		TCCR1B = 0;
	}
}

/*
      A1 A2 A3 A4 A5
  K1   2  3  4  5  6
  K2  10 20 40  7  8
  K3   1 30 50 --  9
*/
void eiuhr_set_leds (uint8_t kathode)
{
	static uint8_t skip;
    
	uint8_t  z, zehner = eiuhr.bits_zehner;
	uint16_t e, einer  = eiuhr.bits_einer;
    
	#define SET_10(b,p) if (z = zehner & (1 << (b/10-1)), z) SET(PORT_ ## p)
	#define SET_1(b,p)  if (e = einer  & (1 << (b-1)), e)    SET(PORT_ ## p)
	
	if (0 == kathode)
		if (++skip == 3)
			skip = 0;
            
	if (!eiuhr.hell && skip != kathode)
        return;			
	
	if (0 == kathode)
	{
		SET_1 (2, A1);
		SET_1 (3, A2);
		SET_1 (4, A3);
		SET_1 (5, A4);
		SET_1 (6, A5);
		
		CLR (PORT_K1);
	}
	
	if (1 == kathode)
	{
		SET_10 (10, A1);
		SET_10 (20, A2);
		SET_10 (40, A3);
		SET_1  ( 7, A4);
		SET_1  ( 8, A5);
		
		CLR (PORT_K2);
	}
	
	if (2 == kathode)
	{
		SET_1  ( 1, A1);
		SET_10 (30, A2);
		SET_10 (50, A3);
        //
		SET_1  ( 9, A5);
		
		CLR (PORT_K3);
	}
}

#include "eiuhr.c"
